#include "board_diag.h"
#include "DM9000A.H"

extern void			iow(unsigned int, unsigned int);
extern unsigned int	ior(unsigned int);
extern void			DM9000_init(void);
extern void			TransmitPacket(unsigned char*, unsigned int );
extern void			ReceivePacket (unsigned char*, unsigned int*);

static void chip_erase(void);
static void program_byte(int, int);

static void TestFlash(void);
static void TestSDRAM(void);
static void TestOutputs(void);
static int Flash_TestDataBus(unsigned int);
static int SDRAM_TestDataBus(unsigned int);
static int read_back_test(unsigned int, int);
static int SDRAM_Test8_16_32BitAccess(unsigned int);
static int SDRAM_TestDevice(unsigned int, unsigned int);
static int Flash_TestAddressBus(unsigned int, unsigned int);
static int SDRAM_TestAddressBus(unsigned int, unsigned int);

int pio_base[] = {PIO_0_BASE, PIO_1_BASE, PIO_2_BASE, PIO_3_BASE, PIO_4_BASE, PIO_5_BASE};
int leds=~0;

//***************************************************************************************
//* main function
//***************************************************************************************
int main()
{
	IOWR_ALTERA_AVALON_PIO_DATA(LEDS_BASE, leds);	// clear LEDs
    DM9000_init();
    DM9000_init();
	TestSDRAM();
	TestFlash();
	TestOutputs();
}

//***************************************************************************************
//* Funktion: TestOutputs
//***************************************************************************************
static void TestOutputs(void)
{
	int i, j;
	int shift_reg;

	// running light for 167 output pins
	printf("Check the LED bands for continuous running light!\n");
	do
	{
		for(i=0; i<6; i++)
		{
			IOWR_ALTERA_AVALON_PIO_DATA(pio_base[i], ~0);
		}

		for(i=0; i<5; i++)
		{
			shift_reg = 1;
			for(j=0; j<=31; j++)
			{
				IOWR_ALTERA_AVALON_PIO_DATA(pio_base[i], ~shift_reg);
				shift_reg = shift_reg << 1;
				msleep(100);
			}
			IOWR_ALTERA_AVALON_PIO_DATA(pio_base[i], ~0);
		}

		shift_reg = 1;
		for(j=0; j<=6; j++)
		{
			IOWR_ALTERA_AVALON_PIO_DATA(pio_base[i], ~shift_reg);
			shift_reg = shift_reg << 1;
			msleep(100);
		}
		IOWR_ALTERA_AVALON_PIO_DATA(pio_base[i], ~0);
	}
	while(1);
}

//***************************************************************************************
//*  Function: TestSDRAM
//*
//*  Purpose: Performs a full-test on the SDRAM.  The tests run are:
//*             - SDRAM_TestDataBus
//*             - SDRAM_TestAddressBus
//*             - SDRAM_Test8_16_32BitAccess
//*             - SDRAM_TestDevice
//*
//***************************************************************************************
static void TestSDRAM(void)
{
	int memory_base = SDRAM_BASE;
	int memory_size = SDRAM_SPAN;

	int ret_code = 0x0;

	printf("\nTesting SDRAM from 0x%X to 0x%X\n", memory_base, (memory_base + memory_size));

	// test SDRAM data bus.
	ret_code = SDRAM_TestDataBus(memory_base);
	if(ret_code) { printf(" - data bus test failed at bit 0x%X\n", (int)ret_code); }
	else
	{		
		printf(" - data bus test passed\n");

		// test SDRAM address bus. 
		ret_code  = SDRAM_TestAddressBus(memory_base, memory_size);
		if(ret_code) { printf(" - address bus test failed at address 0x%X\n", (int)ret_code); }
		else
		{
			printf(" - address bus test passed\n"); }
			
			// test byte and half-word access.
			ret_code = SDRAM_Test8_16_32BitAccess(memory_base);
			if(ret_code) { printf(" - byte, half-word and word access test failed at address 0x%X\n", (int)ret_code); }
			else { printf(" - byte, half-word and word access test passed\n");
		}
	}

	// Test that each bit in the device can store both 1 and 0.
	if (!ret_code)
	{
		printf(" - testing each bit in memory device");
		ret_code = SDRAM_TestDevice(memory_base, memory_size);
		if(ret_code) { printf("  failed at address 0x%X\n", (int)ret_code); }
		else { printf("  passed\n"); }
	}

	if (!ret_code)
	{
		printf("   SDRAM memory at 0x%X okay\n", memory_base);
		leds = leds & ~1;
		IOWR_ALTERA_AVALON_PIO_DATA(LEDS_BASE, leds);	// set LED #0
	}
}

//***************************************************************************************
//* Function: SDRAM_TestDataBus
//*
//* Purpose:
//*	Tests that the data bus is connected with no stuck-at's, shorts, or open circuits.
//*
//***************************************************************************************
static int SDRAM_TestDataBus(unsigned int address)
{
	int i;
	unsigned int pattern;
	unsigned int ret_code = 0x0;

	// Perform a walking 1's test at the given address.
	for(i=0; i<=4; i+=4)
	{
		// test the least significant 32 data bits at first run
		// test the most significant 32 data bits at second run
		for(pattern = 1; pattern != 0; pattern <<= 1)
		{
			IOWR_32DIRECT(address+i, 0, pattern);		// write the test pattern
			if(IORD_32DIRECT(address+i, 0)!=pattern)	// read it back (immediately is okay for this test)
			{
				ret_code = pattern;
				break;
			}
		}
	}
	return ret_code;
}

//***************************************************************************************
//* Function: SDRAM_TestAddressBus
//*
//* Purpose:
//* Tests that the address bus is connected with no stuck-at's, shorts, or open circuits.
//*
//***************************************************************************************
static int SDRAM_TestAddressBus(unsigned int memory_base, unsigned int nBytes)
{
	unsigned int address_mask = (nBytes - 1);
	unsigned int offset;
	unsigned int test_offset;

	unsigned int pattern     = 0xAAAAAAAA;
	unsigned int antipattern  = 0x55555555;

	unsigned int ret_code = 0x0;

	// Write the default pattern at each of the power-of-two offsets
	for(offset = sizeof(unsigned int); (offset & address_mask) != 0; offset <<= 1)
	{
		IOWR_32DIRECT(memory_base, offset, pattern);
	}

	// Check for address bits stuck high
	test_offset = 0;
	IOWR_32DIRECT(memory_base, test_offset, antipattern);
	for (offset = sizeof(unsigned int); (offset & address_mask) != 0; offset <<= 1)
	{
		if (IORD_32DIRECT(memory_base, offset) != pattern)
		{
			ret_code = (memory_base+offset);
			break;
		}
	}

	// Check for address bits stuck low or shorted
	IOWR_32DIRECT(memory_base, test_offset, pattern);
	for (test_offset = sizeof(unsigned int); (test_offset & address_mask) != 0; test_offset <<= 1)
	{
		if (!ret_code)
		{
			IOWR_32DIRECT(memory_base, test_offset, antipattern);
			for (offset = sizeof(unsigned int); (offset & address_mask) != 0; offset <<= 1)
			{
				if ((IORD_32DIRECT(memory_base, offset) != pattern) && (offset != test_offset))
				{
					ret_code = (memory_base + test_offset);
					break;
				}
			}
		IOWR_32DIRECT(memory_base, test_offset, pattern);
		}
	}
	return ret_code;
}

//***************************************************************************************
//*  Function: MemTest8_16BitAccess
//*
//*  Purpose: Tests that the memory at the specified base address
//*           can be read and written in both byte and half-word modes.
//*
//***************************************************************************************
static int SDRAM_Test8_16_32BitAccess(unsigned int memory_base)
{
	int i, ret_code = 0x0, pattern;

	for(i=0; i<=4 && !ret_code; i+=4)
	{
		// test the least significant 32 data bits at first run
		// test the most significant 32 data bits at second run

		pattern = 0x50A0050A;
		// write 4 bytes
		IOWR_8DIRECT(memory_base+i, 0, pattern & 0xff);
		IOWR_8DIRECT(memory_base+i, 1, (pattern >> 8) & 0xff);
		IOWR_8DIRECT(memory_base+i, 2, (pattern >> 16) & 0xff);
		IOWR_8DIRECT(memory_base+i, 3, (pattern >> 24) & 0xff);

		// read it back as one word, as 2 half-words and as 4 bytes
		ret_code = read_back_test(memory_base+i, pattern);

		// write 2 half-words
		if (!ret_code)
		{
			pattern = 0x30C0030C;
			IOWR_16DIRECT(memory_base+i, 0, pattern & 0xffff);
			IOWR_16DIRECT(memory_base+i, 2, (pattern >> 16) & 0xffff);

			// read it back as one word, as 2 half-words and as 4 bytes
			ret_code = read_back_test(memory_base+i, pattern);
		}

		// write one word
		if (!ret_code)
		{
			pattern = 0x050A50A0;
			IOWR_32DIRECT(memory_base+i, 0, pattern);

			// read it back as one word, as 2 half-words and as 4 bytes
			ret_code = read_back_test(memory_base+i, pattern);
		}
	}
	return(ret_code);
}

//***************************************************************************************
// subfunction from SDRAM_Test8_16_32BitAccess
static int read_back_test(unsigned int memory_base, int pattern)
{
	int ret_code = 0x0;
	
	// read it back as one word
	if(IORD_32DIRECT(memory_base, 0) != pattern)
	{
		ret_code = memory_base;
	}

	// read it back as two half-words
	if (!ret_code)
	{
		if ((IORD_16DIRECT(memory_base, 2) != ((pattern >> 16) & 0xffff)) && (IORD_16DIRECT(memory_base, 0) != (pattern & 0xffff)))
		{
			ret_code = memory_base;
		}
	}

	// read it back as 4 bytes
	if (!ret_code)
	{
		if ((IORD_8DIRECT(memory_base, 3) != ((pattern >> 24) & 0xff)) && (IORD_8DIRECT(memory_base, 2) != ((pattern >> 16) & 0xff)) &&
			(IORD_8DIRECT(memory_base, 1) != ((pattern >> 8) & 0xff)) && (IORD_8DIRECT(memory_base, 0) != (pattern & 0xff)))
		{
			ret_code = memory_base;
		}
	}
	return(ret_code);
}

//***************************************************************************************
//*  Function: SDRAM_TestDevice
//*
//*  Purpose: Tests that every bit in the memory device within the 
//*           specified address range can store both a '1' and a '0'.
//*
//***************************************************************************************
static int SDRAM_TestDevice(unsigned int memory_base, unsigned int nBytes)
{
	unsigned int offset;
	unsigned int pattern;
	unsigned int ret_code = 0x0;

	// fill memory with a known pattern
	for (pattern = 1, offset = 0; offset < nBytes; pattern++, offset+=4)
	{
		IOWR_32DIRECT(memory_base, offset, pattern);
	}

	printf(" .");

	// Check each location and invert it for the second pass.
	for (pattern = 1, offset = 0; offset < nBytes; pattern++, offset+=4)
	{
		if (IORD_32DIRECT(memory_base, offset) != pattern)
		{
			ret_code = (memory_base + offset);
			break;
		}
	IOWR_32DIRECT(memory_base, offset, ~pattern);
	}

	printf(" .");

	// Check each location for the inverted pattern and zero it
	for (pattern = 1, offset = 0; offset < nBytes; pattern++, offset+=4)
	{
		if (IORD_32DIRECT(memory_base, offset) != ~pattern)
		{
			ret_code = (memory_base + offset);
			break;
		}
		IOWR_32DIRECT(memory_base, offset, 0x0);
	}
	return ret_code;
}


//***************************************************************************************
//*  Function: TestFlash
//*
//*  Purpose: Performs a full-test on the SDRAM.  The tests run are:
//*             - SDRAM_TestDataBus
//*             - SDRAM_TestAddressBus
//*             - SDRAM_Test8_16_32BitAccess
//*             - SDRAM_TestDevice
//*
//***************************************************************************************
static void TestFlash(void)
{
	int memory_base = FLASH_BASE;
	int memory_size = FLASH_SPAN;

	int ret_code = 0x0;

	printf("\nTesting Flash from 0x%X to 0x%X\n", memory_base, (memory_base + memory_size));
	printf(" - erasing flash memory ... ");
	chip_erase();
	printf("done\n");

	// test Flash data bus.
	ret_code = Flash_TestDataBus(memory_base);
	if(ret_code) { printf(" - data bus test failed at bit 0x%X\n", (int)ret_code); }
	else
	{		
		printf(" - data bus test passed\n");

		// test Flash address bus. 
		ret_code  = Flash_TestAddressBus(memory_base, memory_size);
		if(ret_code) { printf(" - address bus test failed at address 0x%X\n", (int)ret_code); }
		else
		{
			printf(" - address bus test passed\n");
		}
	}

	if (!ret_code)
	{
		printf("   Flash memory at 0x%X okay\n", memory_base);
		leds = leds & ~2;
		IOWR_ALTERA_AVALON_PIO_DATA(LEDS_BASE, leds);	// set LED #1
	}
}

//***************************************************************************************
//* Function: Flash_TestDataBus
//*
//* Purpose:
//*	Tests that the data bus is connected with no stuck-at's, shorts, or open circuits.
//*
//***************************************************************************************
static int Flash_TestDataBus(unsigned int address)
{
	int addr = 0x8010;
	unsigned int pattern;
	unsigned int ret_code = 0x0;

	// test 8 data bits
	for(pattern = 1; (pattern & 0xff) != 0; pattern <<= 1)
	{
		program_byte(addr, pattern);
		if(IORD_8DIRECT(FLASH_BASE,addr++) != pattern)	// read it back
		{
			ret_code = pattern;
			break;
		}

		program_byte(addr, (~pattern & 0xff));
		if(IORD_8DIRECT(FLASH_BASE,addr++) != (~pattern & 0xff))	// read it back
		{
			ret_code = (~pattern & 0xff);
			break;
		}

	}
	return ret_code;
}

//***************************************************************************************
//* Function: Flash_TestAddressBus
//*
//* Purpose:
//* Tests that the address bus is connected with no stuck-at's, shorts, or open circuits.
//*
//***************************************************************************************
static int Flash_TestAddressBus(unsigned int memory_base, unsigned int nBytes)
{
	int address_mask;
	int pattern = 0xAA;
	int ret_code = 0x0;

	// test 24 address bits
	for(address_mask=nBytes>>1; address_mask != 0; address_mask >>= 1)
	{
		program_byte(address_mask, pattern);
		if(IORD_8DIRECT(FLASH_BASE, address_mask) != pattern)	// read it back
		{
			ret_code = address_mask;
			break;
		}
		program_byte(address_mask, 0);

		program_byte((~address_mask & (nBytes-1)), pattern);
		if(IORD_8DIRECT(FLASH_BASE, (~address_mask & (nBytes-1))) != pattern)	// read it back
		{
			ret_code = ~address_mask & (nBytes-1);
			break;
		}
		program_byte((~address_mask & (nBytes-1)), 0);
	}
	return ret_code;
}

//***************************************************************************************
// programs byte "data" in flash at address "flash_addr"
//
static void program_byte(int flash_addr, int data)
{
	IOWR_8DIRECT(FLASH_BASE, 0x0AAA, 0xAA);					// word program command sequence
	IOWR_8DIRECT(FLASH_BASE, 0x0555, 0x55);
	IOWR_8DIRECT(FLASH_BASE, 0x0AAA, 0xA0);
	IOWR_8DIRECT(FLASH_BASE, flash_addr, data);

	while(data != IORD_8DIRECT(FLASH_BASE, flash_addr));	// wait until byte is programmed
}

//***************************************************************************************
// sets all bits in flash memory to '1'
//
static void chip_erase(void)
{
	IOWR_8DIRECT(FLASH_BASE, 0x0AAA, 0xAA);					// chip erase command sequence
	IOWR_8DIRECT(FLASH_BASE, 0x0555, 0x55);
	IOWR_8DIRECT(FLASH_BASE, 0x0AAA, 0x80);
	IOWR_8DIRECT(FLASH_BASE, 0x0AAA, 0xAA);
	IOWR_8DIRECT(FLASH_BASE, 0x0555, 0x55);
	IOWR_8DIRECT(FLASH_BASE, 0x0AAA, 0x10);
	
	while( 0xff != IORD_8DIRECT(FLASH_BASE, 0));			// wait until chip is erased
}

